@@ -4,31 +4,51 @@ __author__  = 'Taito Horiuchi <taito.horiuchi@abita.fi>'
 __docformat__ = 'plaintext'
 
 import md5
+from email import Encoders
+from email.MIMEMultipart import MIMEMultipart
+from email.MIMEText import MIMEText
+from email.MIMEAudio import MIMEAudio
+from email.MIMEBase import MIMEBase
+from email.MIMEImage import MIMEImage
 from Acquisition import aq_parent
 
 from Products.CMFCore.utils import getToolByName
-from Products.Archetypes.public import DisplayList, registerType, Schema
+from Products.Archetypes.public import (
+#    ATFieldProperty,
+    DisplayList,
+    Schema,
+#    registerType,
+)
+from Products.ATContentTypes.content.base import registerATCT
+from Products.ATContentTypes.content.folder import ATFolderSchema, ATFolder
 from Products.ATContentTypes.configuration import zconf
-from Products.Archetypes.public import RFC822Marshaller
+#from Products.Archetypes.public import RFC822Marshaller
 
 
 from Products.ATContentTypes.content.document import finalizeATCTSchema
 
 from Products.PloneFormGen.content.actionAdapter import (
     AnnotationStorage,
-    FormActionAdapter,
-    FormAdapterSchema,
+#    FormActionAdapter,
+#    FormAdapterSchema,
+    BooleanField,
     IntegerField,
+    LinesField,
     StringField,
     TextField,
+    BooleanWidget,
     IntegerWidget,
+    MultiSelectionWidget,
     RichWidget,
     SelectionWidget,
     StringWidget,
-    TextAreaWidget,
+#    TextAreaWidget,
 )
-from Products.PloneFormGen.content.formMailerAdapter import FormMailerAdapter, formMailerAdapterSchema
+from Products.PloneFormGen.content.formMailerAdapter import formMailerAdapterSchema, FormMailerAdapter
+from Products.PloneFormGen import HAS_PLONE25, HAS_PLONE30
+
 from Products.PFGVerkkomaksut import PFGVerkkomaksutMessageFactory as _
+from Products.PFGVerkkomaksut.config import PROJECTNAME
 
 def check_float(value):
     try:
@@ -39,16 +59,47 @@ def check_float(value):
     except TypeError:
         return False
 
-#PFGVerkkomaksutSchema = FormAdapterSchema.copy() + Schema((
-PFGVerkkomaksutSchema = formMailerAdapterSchema + Schema((
+PFGVerkkomaksutSchema = ATFolderSchema.copy() + formMailerAdapterSchema.copy() + Schema((
+
+    BooleanField(
+        name='msg_necessary',
+        schemata='message',
+        required=False,
+        searchable=False,
+        languageIndependent=True,
+        storage=AnnotationStorage(),
+        widget=BooleanWidget(
+            label=_(u'Message Necessary'),
+            description=_(u'If message after success of form input is necessary to send, please check this box.'),
+        ),
+    ),
+
+    LinesField(
+#        name='attachments_before_verkkomaksut',
+        name='msg_attachments',
+        schemata='message',
+        required=False,
+        searchable=False,
+        languageIndependent=True,
+        storage=AnnotationStorage(),
+        widget=MultiSelectionWidget(
+            label=_(u'E-mail attachments before Verkkomaksut'),
+            description=_(u'Please select the attachments to be sent with email when one has successfully finished inputs of the form.'),
+            format='checkbox',
+        ),
+        vocabulary='attachments',
+        enforceVocabulary=True,
+    ),
+
     IntegerField(
         name='merchant_id',
+        schemata='verkkomaksut',
         required=True,
         searchable=False,
         storage=AnnotationStorage(),
         widget=IntegerWidget(
             label=_(u'Merchant ID'),
-            description=_(u'Please imput MERCHANT_ID here.'),
+            description=_(u'Please input MERCHANT_ID provided by Verkkomaksut here. The default ID is for demo use which is 13466.'),
             maxlength=11,
         ),
         size=11,
@@ -57,12 +108,13 @@ PFGVerkkomaksutSchema = formMailerAdapterSchema + Schema((
 
     StringField(
         name='merchant_authentication_code',
+        schemata='verkkomaksut',
         required=True,
         searchable=False,
         storage=AnnotationStorage(),
             widget=StringWidget(
                 label=_(u'Merchant Authentication Code'),
-                description=_(u''),
+                description=_(u'Please input Merchant Authentication Code provided by Verkkomaksut here. The default ID is for demo use which is 6pKF4jkv97zmqBJ3ZL8gUw5DfT2NMQ.'),
                 maxlength=32,
             ),
         default='6pKF4jkv97zmqBJ3ZL8gUw5DfT2NMQ'
@@ -70,12 +122,13 @@ PFGVerkkomaksutSchema = formMailerAdapterSchema + Schema((
 
     IntegerField(
         name='next_order_number',
+        schemata='verkkomaksut',
         required=True,
         searchable=False,
         storage=AnnotationStorage(),
         widget=IntegerWidget(
             label=_(u'Next Order Number'),
-            description=_(u'Please imput the next order number here. The incremental numbers are used.'),
+            description=_(u'Please input the next order number here. This number will be automatically increased by one for each transaction and will be sent to Verkkomksut.'),
             maxlength=64,
         ),
         size=10,
@@ -84,91 +137,100 @@ PFGVerkkomaksutSchema = formMailerAdapterSchema + Schema((
 
     StringField(
         name='price_field',
+        schemata='verkkomaksut',
         required=True,
         searchable=False,
         languageIndependent=True,
         storage=AnnotationStorage(),
         widget=SelectionWidget(
             label=_(u'Price Field'),
-            description=_(u'Please select the price field.'),
+            description=_(u'Please select the price field which will be used for Verkkomakusut. Remember to add fixed-point field or selection field with required option checked to the form folder first.'),
         ),
         vocabulary='price_fields',
         enforceVocabulary=True,
     ),
 
-    TextField('thanks_text',
-              required=False,
-              searchable=True,
-#              primary=True,
-              storage = AnnotationStorage(),
-#              validators = ('isTidyHtmlWithCleanup',),
-#              default_output_type = 'text/x-html-safe',
-              widget = TextAreaWidget(
-#              widget = RichWidget(
-                        label = _(u'Thanks Text which will be displayed when payment is successful.'),
-                        rows = 5,
+    TextField(
+        name='thanks_text',
+        schemata='verkkomaksut',
+        required=False,
+        searchable=False,
+        storage = AnnotationStorage(),
+        primary=True,
+        validators = ('isTidyHtmlWithCleanup',),
+        default_output_type = 'text/x-html-safe',
+#        widget = TextAreaWidget(
+        widget = RichWidget(
+            label = _(u'Thanks Text'),
+            description=_(u'This thanks text will be displayed when payment is successful.'),
+            rows = 25,
+            allow_file_upload = zconf.ATDocument.allow_document_upload,
               ),
     ),
 
-#    TextField('thanks_text',
-##        schemata='decoration',
-#        required=False,
-#        searchable=False,
-#        primary=False,
-#        validators = ('isTidyHtmlWithCleanup',),
-##        default_content_type = zconf.ATDocument.default_content_type,
-#        default_output_type = 'text/x-html-safe',
-##        allowable_content_types = zconf.ATDocument.allowed_content_types,
-#        widget = RichWidget(
-#            label = "Thanks Text",
-##            label_msgid = "label_thanksepilogue_text",
-#            description = "Thanks Text which will be displayed when payment is successful.",
-##            description_msgid = "help_thanksepilogue_text",
-#            rows = 8,
-#            i18n_domain = "pfg.verkkomaksut",
-##            allow_file_upload = zconf.ATDocument.allow_document_upload,
-#            ),
-#        ),
-
-#    TextField('text',
-#              required=True,
-#              searchable=True,
-#              primary=True,
-#              storage = AnnotationStorage(migrate=True),
-#              validators = ('isTidyHtmlWithCleanup',),
-#              #validators = ('isTidyHtml',),
-#              default_content_type = zconf.ATDocument.default_content_type,
-#              default_output_type = 'text/x-html-safe',
-#              allowable_content_types = zconf.ATDocument.allowed_content_types,
-#              widget = RichWidget(
-#                        description = "",
-#                        description_msgid = "help_body_text",
-#                        label = "Body Text",
-#                        label_msgid = "label_body_text",
-#                        rows = 25,
-#                        i18n_domain = "plone",
-#                        allow_file_upload = zconf.ATDocument.allow_document_upload)),
+    StringField(
+        name='cancel_message',
+        schemata='verkkomaksut',
+        required=False,
+        searchable=False,
+        storage=AnnotationStorage(),
+            widget=StringWidget(
+                label=_(u'Cancel Message'),
+                description=_(u'This message will be shown when one cancels Verkkomaksut payment.'),
+                maxlength=255,
+            ),
+        ),
 
     ),)
 
-finalizeATCTSchema(PFGVerkkomaksutSchema)
+PFGVerkkomaksutSchema.moveField('msg_necessary', pos='top')
+PFGVerkkomaksutSchema.moveField('body_pt', before='body_pre')
+PFGVerkkomaksutSchema['body_pt'].schemata='message'
+PFGVerkkomaksutSchema['body_pt'].required=False
+#PFGVerkkomaksutSchema['msg_subject'].storage=AnnotationStorage()
 
-class PFGVerkkomaksut(FormActionAdapter, FormMailerAdapter):
+#finalizeATCTSchema(PFGVerkkomaksutSchema)
+finalizeATCTSchema(PFGVerkkomaksutSchema, folderish=True, moveDiscussion=False)
+
+#class PFGVerkkomaksut(FormActionAdapter, FormMailerAdapter):
+#class PFGVerkkomaksut(FormMailerAdapter):
+class PFGVerkkomaksut(ATFolder, FormMailerAdapter):
+#class PFGVerkkomaksut(FormMailerAdapter, ATFolder):
     """Verkkomaksut Payment Adapter"""
 
     # Standard content type setup
     portal_type = meta_type = 'PFGVerkkomaksut'
     archetype_name = 'Verkkomaksut Payment Adapter'
     schema = PFGVerkkomaksutSchema
-    typeDescription = 'A document which can contain rich text, images and attachments'
-    typeDescMsgId = 'PFGVerkkomaksut_description_edit'
+#    typeDescription = 'A document which can contain rich text, images and attachments'
+#    typeDescMsgId = 'PFGVerkkomaksut_description_edit'
+
+    default_view = immediate_view = 'base_view'
 
     _at_rename_after_creation = True
 
+    __implements__ = (ATFolder.__implements__, )
+
+
+#    merchant_id = ATFieldProperty('merchant_id')
+#    merchant_authentication_code = ATFieldProperty('merchant_authentication_code')
+#    next_order_number = ATFieldProperty('next_order_number')
+#    price_field = ATFieldProperty('price_field')
+#    thanks_text = ATFieldProperty('thanks_text')
+#    cancel_message = ATFieldProperty('cancel_message')
+
     def canSetDefaultPage(self):
         return False
 
     def onSuccess(self, fields, REQUEST=None):
+        order_number = self.getNext_order_number()
+        ORDER_NUMBER = str(order_number)
+        # Increase next_order_number by one.
+        self.setNext_order_number(order_number + 1)
+        REQUEST.set('ORDER_NUMBER', ORDER_NUMBER)
+        if self.getMsg_necessary():
+#            super(PFGVerkkomaksut, self).onSuccess(fields, REQUEST)
+            self.send_form(fields, REQUEST)
         field = self.selected_price_field()
         form = self.REQUEST.form
         price = form.get(field)
@@ -180,8 +242,8 @@ class PFGVerkkomaksut(FormActionAdapter, FormMailerAdapter):
         self.setNext_order_number(order_number + 1)
         ORDER_DESCRIPTION = 'Payment from PloneFormGen Verkkomaksut Adapter'
         CURRENCY = 'EUR'
-        portal = getToolByName(self, 'portal_url').getPortalObject()
-        portal_url = portal.absolute_url()
+#        portal = getToolByName(self, 'portal_url').getPortalObject()
+#        portal_url = portal.absolute_url()
         parent = aq_parent(self)
 #        ids = parent.objectIds()
 #        ids = [id for id in ids if id in form.keys()]
@@ -253,6 +315,92 @@ class PFGVerkkomaksut(FormActionAdapter, FormMailerAdapter):
         url = '%s/verkkomaksut' % parent_url
         return self.REQUEST.RESPONSE.redirect(url)
 
+    def send_form(self, fields, request, **kwargs):
+        """Send the form.
+        """
+        (headerinfo, additional_headers, body) = self.get_header_body_tuple(fields, request, **kwargs)
+        if not isinstance(body, unicode):
+            body = unicode(body, self._site_encoding())
+        portal = getToolByName(self, 'portal_url').getPortalObject()
+        email_charset = portal.getProperty('email_charset', 'utf-8')
+        mime_text = MIMEText(body.encode(email_charset , 'replace'),
+                _subtype=self.body_type or 'html', _charset=email_charset)
+
+        attachments = self.get_attachments(fields, request)
+        uids = self.getMsg_attachments()
+        if uids:
+            reference_catalog = getToolByName(self, 'reference_catalog')
+            for uid in uids:
+                obj = reference_catalog.lookupObject(uid)
+                data = obj.data
+                mimetype = obj.content_type
+                filename = obj.getRawTitle()
+                enc = None
+                attachments.append((filename, mimetype, enc, data))
+
+        if attachments:
+            outer = MIMEMultipart()
+            outer.attach(mime_text)
+        else:
+            outer = mime_text
+
+        # write header
+        for key, value in headerinfo.items():
+            outer[key] = value
+
+        # write additional header
+        for a in additional_headers:
+            key, value = a.split(':', 1)
+            outer.add_header(key, value.strip())
+
+        for attachment in attachments:
+            filename = attachment[0]
+            ctype = attachment[1]
+            encoding = attachment[2]
+            content = attachment[3]
+
+            if ctype is None:
+                ctype = 'application/octet-stream'
+
+            maintype, subtype = ctype.split('/', 1)
+
+            if maintype == 'text':
+                msg = MIMEText(content, _subtype=subtype)
+            elif maintype == 'image':
+                msg = MIMEImage(content, _subtype=subtype)
+            elif maintype == 'audio':
+                msg = MIMEAudio(content, _subtype=subtype)
+            else:
+                msg = MIMEBase(maintype, subtype)
+                msg.set_payload(content)
+                # Encode the payload using Base64
+                Encoders.encode_base64(msg)
+
+            # Set the filename parameter
+            msg.add_header('Content-Disposition', 'attachment', filename=filename)
+            outer.attach(msg)
+
+        mailtext = outer.as_string()
+
+        host = self.MailHost
+        host.send(mailtext)
+
+
+    def attachments(self):
+        dl = DisplayList()
+        catalog = getToolByName(self, 'portal_catalog')
+        path = '/'.join(self.getPhysicalPath())
+        brains = catalog(
+            portal_type=('File','Image',),
+            path=dict(query=path, depth=1),
+        )
+        for brain in brains:
+            if HAS_PLONE25 and not HAS_PLONE30:
+                dl.add(brain.getObject().UID(), brain.Title)
+            if HAS_PLONE30:
+                dl.add(brain.UID, brain.Title)
+        return dl
+
     def price_fields(self):
         dl = DisplayList()
         catalog = getToolByName(self, 'portal_catalog')
@@ -284,7 +432,5 @@ class PFGVerkkomaksut(FormActionAdapter, FormMailerAdapter):
         obj = reference_catalog.lookupObject(uid)
         return obj.getId()
 
-    def success(self):
-        return 'AA'
-
-registerType(PFGVerkkomaksut)
+#registerType(PFGVerkkomaksut)
+registerATCT(PFGVerkkomaksut, PROJECTNAME)
